@@ -37,6 +37,7 @@
   * System library.
   */
 #include <sys_defs.h>
+#include <string.h>
 
  /*
   * Utility library.
@@ -59,16 +60,29 @@ void    config_known_tcp_ports(const char *source, const char *settings)
     ARGV   *associations;
     ARGV   *association;
     char  **cpp;
+    char   *copy;
+    char   *cp;
 
     clear_known_tcp_ports();
 
+    /*
+     * We can strip comments before splitting the input.
+     */
+    copy = mystrdup(settings);
+    if ((cp = strchr(copy, '#')) != 0
+	&& (cp == copy || strchr(CHARS_COMMA_SP "=", cp[-1]) != 0)) {
+	msg_warn("%s: #comment after other text is not allowed: %s", source, cp);;
+	*cp = 0;
+    }
+
     /*
      * The settings is in the form of associations separated by comma. Split
      * it into separate associations.
      */
-    associations = argv_split(settings, ",");	/* blame later */
+    associations = argv_split(copy, ",");
     if (associations->argc == 0) {
 	argv_free(associations);
+	myfree(copy);
 	return;
     }
 
@@ -81,14 +95,16 @@ void    config_known_tcp_ports(const char *source, const char *settings)
     for (cpp = associations->argv; *cpp != 0; cpp++) {
 	char   *temp = concatenate(" ", *cpp, " ", (char *) 0);
 
-	association = argv_split_at(temp, '=');	/* blame later */
+	association = argv_split_at(temp, '=');
 	myfree(temp);
 
 	if (association->argc == 0) {
 	     /* empty, ignore */ ;
 	} else if (association->argc == 1) {
-	    msg_warn("%s: in \"%s\" is not in \"name = value\" form",
-		     source, *cpp);
+	    cp = *cpp;
+	    if ((cp = mystrtok(&cp, CHARS_SPACE)) != 0)
+		msg_warn("%s: \"%s\" is not in \"name = value\" form",
+			 source, cp);
 	} else {
 	    char   *bp;
 	    char   *lhs;
@@ -99,8 +115,6 @@ void    config_known_tcp_ports(const char *source, const char *settings)
 	    bp = association->argv[association->argc - 1];
 	    if ((rhs = mystrtok(&bp, CHARS_SPACE)) == 0) {
 		err = "missing port value after \"=\"";
-	    } else if (*rhs == '#') {
-		err = "#comment after other text is not allowed";
 	    } else if (mystrtok(&bp, CHARS_SPACE) != 0) {
 		err = "whitespace in port number";
 	    } else {
@@ -110,8 +124,6 @@ void    config_known_tcp_ports(const char *source, const char *settings)
 		    bp = association->argv[n];
 		    if ((lhs = mystrtok(&bp, CHARS_SPACE)) == 0) {
 			new_err = "missing service name before \"=\"";
-		    } else if (*lhs == '#') {
-			err = "#comment after other text is not allowed";
 		    } else if (mystrtok(&bp, CHARS_SPACE) != 0) {
 			new_err = "whitespace in service name";
 		    } else {
@@ -128,6 +140,7 @@ void    config_known_tcp_ports(const char *source, const char *settings)
 	argv_free(association);
     }
     argv_free(associations);
+    myfree(copy);
 }
 
 #ifdef TEST
@@ -164,6 +177,34 @@ static struct test_case test_cases[] = {
 	 /* warning */ "",
 	 /* export */ "lmtp=24 smtp=25 smtps=465 submissions=465"
     },
+    {"comment 1",
+	 /* config */ "smtp = 25,#smtps = submissions = 465, lmtp = 24",
+	 /* warning */ "config_known_tcp_ports: warning: comment 1: #comment after other text is not allowed: #smtps = submissions = 465, lmtp = 24\n",
+	 /* export */ "smtp=25"
+    },
+    {"comment 2",
+	 /* config */ "smtp = 25, smtps #= submissions = 465, lmtp = 24",
+	 /* warning */ "config_known_tcp_ports: warning: comment 2: #comment after other text is not allowed: #= submissions = 465, lmtp = 24\n"
+	"config_known_tcp_ports: warning: comment 2: \"smtps\" is not in \"name = value\" form\n",
+	 /* export */ "smtp=25"
+    },
+    {"comment 3",
+	 /* config */ "smtp = 25, smtps =#submissions = 465, lmtp = 24",
+	 /* warning */ "config_known_tcp_ports: warning: comment 3: #comment after other text is not allowed: #submissions = 465, lmtp = 24\n"
+	"config_known_tcp_ports: warning: comment 3: in \" smtps =\": missing port value after \"=\"\n",
+	 /* export */ "smtp=25"
+    },
+    {"comment 4",
+	 /* config */ "smtp = 25, smtps = submissions #= 465, lmtp = 24",
+	 /* warning */ "config_known_tcp_ports: warning: comment 4: #comment after other text is not allowed: #= 465, lmtp = 24\n"
+	"config_known_tcp_ports: warning: comment 4: in \" smtps = submissions \": non-numerical service port\n",
+	 /* export */ "smtp=25"
+    },
+    {"comment 5",
+	 /* config */ "smtp = 25, smtps = submissions = 465, #lmtp = 24",
+	 /* warning */ "config_known_tcp_ports: warning: comment 5: #comment after other text is not allowed: #lmtp = 24\n",
+	 /* export */ "smtp=25 smtps=465 submissions=465"
+    },
     {"equal-equal",
 	 /* config */ "smtp = 25, smtps == submissions = 465, lmtp = 24",
 	 /* warning */ "config_known_tcp_ports: warning: equal-equal: "
@@ -223,6 +264,7 @@ int     main(int argc, char **argv)
     msg_buf = vstring_alloc(100);
     for (tp = test_cases; tp->label != 0; tp++) {
 	test_failed = 0;
+	VSTRING_RESET(msg_buf);
 	if ((memory_stream = vstream_memopen(msg_buf, O_WRONLY)) == 0)
 	    msg_fatal("open memory stream: %m");
 	vstream_swap(VSTREAM_ERR, memory_stream);
@@ -242,10 +284,9 @@ int     main(int argc, char **argv)
 			 tp->label, export, tp->exp_export);
 		test_failed = 1;
 	    }
-	    clear_known_tcp_ports();
-	    VSTRING_RESET(msg_buf);
-	    VSTRING_TERMINATE(msg_buf);
 	}
+	VSTRING_RESET(msg_buf);
+	VSTRING_TERMINATE(msg_buf);
 	if (test_failed) {
 	    msg_info("%s: FAIL", tp->label);
 	    fail++;
