@@ -339,7 +339,7 @@ static ARGV *milter_macro_lookup(MILTERS *milters, const char *macro_names)
     const char *cname;
 
     /* XXX Which parameter to blame? */
-    while ((name = mystrtok_cw(&cp, CHARS_COMMA_SP, "milter_macros")) != 0) {
+    while ((name = mystrtok_cw(&cp, CHARS_COMMA_SP, "milter macros")) != 0) {
 	if (msg_verbose)
 	    msg_info("%s: \"%s\"", myname, name);
 	if (*name != '{')			/* } */
