@@ -208,7 +208,7 @@
 /*	values for the private directories of the new instance. The
 /*	"\fB-G \fIgroup\fR" option may be specified to assign the
 /*	instance to a group, otherwise, the new instance is not a
-/*	member of any groups.
+/*	member of any group.
 /* .sp
 /*	The new instance main.cf is the stock main.cf with the
 /*	parameters that specify the locations of shared files cloned
@@ -233,9 +233,9 @@
 /* .RE
 /* .IP
 /*	If any of these pathnames is not supplied, the program
-/*	attempts to generate the pathname by taking the corresponding
-/*	primary instance pathname, and by replacing the last pathname
-/*	component by the value of the \fB-I\fR option.
+/*	attempts to generate the missing pathname(s) by taking the
+/*	corresponding primary instance pathname, and replacing the
+/*	last pathname component by the value of the \fB-I\fR option.
 /* .sp
 /*	If the instance configuration directory already exists, and
 /*	contains both a main.cf and master.cf file, \fBcreate\fR
@@ -326,7 +326,7 @@
 /* .IP "\fBdaemon_directory (see 'postconf -d' output)\fR"
 /*	The directory with Postfix support programs and daemon programs.
 /* .IP "\fBimport_environment (see 'postconf -d' output)\fR"
-/*	The list of environment parameters that a privileged Postfix
+/*	The list of environment variables that a privileged Postfix
 /*	process will import from a non-Postfix parent process, or name=value
 /*	environment overrides.
 /* .IP "\fBmulti_instance_directories (empty)\fR"
@@ -404,6 +404,8 @@
 /*	Google, Inc.
 /*	111 8th Avenue
 /*	New York, NY 10011, USA
+/*
+/*	Wietse Venema
 /*--*/
 
 /* System library. */
@@ -872,7 +874,8 @@ static void load_all_instances(void)
      * only comma characters. Count the actual number of elements, before we
      * decide that the list is empty.
      */
-    secondary_names = argv_split(var_multi_conf_dirs, CHARS_COMMA_SP);
+    secondary_names = argv_split_cw(var_multi_conf_dirs, CHARS_COMMA_SP,
+				    VAR_MULTI_CONF_DIRS);
 
     /*
      * First, the primary instance.  This is synthesized out of thin air.
@@ -1471,14 +1474,15 @@ static int run_user_command(INSTANCE *ip, int iter_cmd, int iter_flags,
 
 /* word_in_list - look up command in start, stop, or control list */
 
-static int word_in_list(char *cmdlist, const char *cmd)
+static int word_in_list(char *cmdlist, const char *cmd, const char *blame)
 {
     char   *saved;
     char   *cp;
     char   *elem;
 
     cp = saved = mystrdup(cmdlist);
-    while ((elem = mystrtok(&cp, CHARS_COMMA_SP)) != 0 && strcmp(elem, cmd) != 0)
+    while ((elem = mystrtok_cw(&cp, CHARS_COMMA_SP, blame)) != 0
+	   && strcmp(elem, cmd) != 0)
 	 /* void */ ;
     myfree(saved);
     return (elem != 0);
@@ -1497,11 +1501,13 @@ static int iterate_postfix_command(int iter_cmd, int argc, char **argv,
     /*
      * Override the iterator controls.
      */
-    if (word_in_list(var_multi_start_cmds, argv[0])) {
+    if (word_in_list(var_multi_start_cmds, argv[0], VAR_MULTI_START_CMDS)) {
 	iter_flags = ITER_FLAG_CHECK_DISABLED;
-    } else if (word_in_list(var_multi_stop_cmds, argv[0])) {
+    } else if (word_in_list(var_multi_stop_cmds, argv[0],
+			    VAR_MULTI_STOP_CMDS)) {
 	iter_flags = ITER_FLAG_SKIP_DISABLED | ITER_FLAG_REVERSE;
-    } else if (word_in_list(var_multi_cntrl_cmds, argv[0])) {
+    } else if (word_in_list(var_multi_cntrl_cmds, argv[0],
+			    VAR_MULTI_CNTRL_CMDS)) {
 	iter_flags = ITER_FLAG_SKIP_DISABLED;
     } else {
 	iter_flags = 0;
