@@ -464,7 +464,7 @@ static int load_chain_files(SSL_CTX *ctx, const char *chain_files)
     pem_load_state_t st;
 
     /* XXX Which parameter to blame? */
-    ARGV   *files = argv_split_cw(chain_files, CHARS_COMMA_SP, "chain_files");
+    ARGV   *files = argv_split_cw(chain_files, CHARS_COMMA_SP, "chain files");
     char  **filep;
     int     ret = 0;
     int     more;
