@@ -187,6 +187,8 @@
 /*	New York, NY 10011, USA
 /*
 /*	Viktor Dukhovni
+/*
+/*	Wietse Venema
 /*--*/
 
 /* System library. */
@@ -392,7 +394,7 @@ static void tlsa_carp(const char *s1, const char *s2, const char *s3,
 	vstring_sprintf(top, "...");
     }
 
-    msg_warn("%s%s%s%s: %u %u %u %s%s%s", s1, s2, s3, s4, u, s, m, STR(top),
+    msg_warn("%s%s%s %s: %u %u %u %s%s%s", s1, s2, s3, s4, u, s, m, STR(top),
 	     dlen > MAX_DUMP_BYTES ? "..." : "",
 	     dlen > MAX_DUMP_BYTES ? STR(bot) : "");
 }
@@ -418,7 +420,8 @@ static void dane_free(void *dane, void *unused_context)
 void    tls_dane_add_fpt_digests(TLS_DANE *dane, const char *digest,
 				         const char *delim, int smtp_mode)
 {
-    ARGV   *values = argv_split(digest, delim);
+    /* XXX Which parameter to blame? */
+    ARGV   *values = argv_split_cw(digest, delim, "fingerprint digests");
     ssize_t i;
 
     if (smtp_mode) {
@@ -807,13 +810,13 @@ int     tls_dane_enable(TLS_SESS_STATE *TLScontext)
 	    continue;
 	}
 	if (ret == 0) {
-	    tlsa_carp(TLScontext->namaddr, ": ", "", "unusable TLSA RR",
+	    tlsa_carp(TLScontext->namaddr, ":", "", "unusable TLSA RR",
 		      tp->usage, tp->selector, tp->mtype, tp->data,
 		      tp->length);
 	    continue;
 	}
 	/* Internal problem in OpenSSL */
-	tlsa_carp(TLScontext->namaddr, ": ", "", "error loading trust settings",
+	tlsa_carp(TLScontext->namaddr, ":", "", "error loading trust settings",
 		  tp->usage, tp->selector, tp->mtype, tp->data, tp->length);
 	tls_print_errors();
 	return (-1);
@@ -823,7 +826,7 @@ int     tls_dane_enable(TLS_SESS_STATE *TLScontext)
 
 /* tls_dane_digest_init - configure supported DANE digests */
 
-void    tls_dane_digest_init(SSL_CTX *ctx, const EVP_MD * fpt_alg)
+void    tls_dane_digest_init(SSL_CTX *ctx, const EVP_MD *fpt_alg)
 {
     dane_mtype mtypes[256];
     char   *cp;
@@ -850,7 +853,8 @@ void    tls_dane_digest_init(SSL_CTX *ctx, const EVP_MD * fpt_alg)
     maxtype = 2;
 
     save = cp = mystrdup(var_tls_dane_digests);
-    while ((algname = mystrtok(&cp, CHARS_COMMA_SP)) != 0) {
+    while ((algname = mystrtok_cw(&cp, CHARS_COMMA_SP,
+				  VAR_TLS_DANE_DIGESTS)) != 0) {
 	char   *algcode = split_at(algname, '=');
 	int     codepoint = -1;
 
@@ -930,7 +934,7 @@ void    tls_dane_digest_init(SSL_CTX *ctx, const EVP_MD * fpt_alg)
 	}
 	mtypes[codepoint].ord = ++ord;
 
-	if ((mtypes[codepoint].alg = EVP_get_digestbyname(algname)) == 0) {
+	if ((mtypes[codepoint].alg = tls_digest_byname(algname, NULL)) == 0) {
 	    msg_warn("%s: digest algorithm \"%s\"(%d) unknown",
 		     VAR_TLS_DANE_DIGESTS, algname, codepoint);
 	    continue;
@@ -1132,11 +1136,11 @@ static void load_tlsa_args(SSL *ssl, char *argv[])
     case 0:
 	break;
     case 1:
-	if ((md = EVP_get_digestbyname(LN_sha256)) == 0)
+	if ((md = tls_digest_byname(LN_sha256, NULL)) == 0)
 	    msg_fatal("Digest %s not found", LN_sha256);
 	break;
     case 2:
-	if ((md = EVP_get_digestbyname(LN_sha512)) == 0)
+	if ((md = tls_digest_byname(LN_sha512, NULL)) == 0)
 	    msg_fatal("Digest %s not found", LN_sha512);
 	break;
     default:
