@@ -156,9 +156,6 @@
 /*	Enable case-insensitive matching.
 /*	This feature is not enabled by default when calling name_mask();
 /*	it has no effect with str_name_mask().
-/* .IP NAME_MASK_NULL
-/*	When converting from mask to string, output "0" when the
-/*	input mask is empty.
 /* .IP NAME_MASK_COMMA
 /*	Use comma instead of space when converting a mask to string.
 /* .IP NAME_MASK_PIPE
@@ -179,6 +176,8 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
 /*--*/
 
 /* System library. */
@@ -231,7 +230,7 @@ int     name_mask_delim_opt(const char *context, const NAME_MASK *table,
      * Break up the names string, and look up each component in the table. If
      * the name is found, merge its mask with the result.
      */
-    while ((name = mystrtok(&bp, delim)) != 0) {
+    while ((name = mystrtok_cw(&bp, delim, context)) != 0) {
 	for (np = table; /* void */ ; np++) {
 	    if (np->name == 0) {
 		if ((flags & NAME_MASK_NUMBER)
@@ -311,8 +310,6 @@ const char *str_name_mask_opt(VSTRING *buf, const char *context,
     }
     if ((len = VSTRING_LEN(buf)) > 0)
 	vstring_truncate(buf, len - 1);
-    else if (flags & NAME_MASK_NULL)
-	vstring_strcat(buf, "0");
     VSTRING_TERMINATE(buf);
 
     return (STR(buf));
@@ -347,7 +344,7 @@ long    long_name_mask_delim_opt(const char *context,
      * Break up the names string, and look up each component in the table. If
      * the name is found, merge its mask with the result.
      */
-    while ((name = mystrtok(&bp, delim)) != 0) {
+    while ((name = mystrtok_cw(&bp, delim, context)) != 0) {
 	for (np = table; /* void */ ; np++) {
 	    if (np->name == 0) {
 		if ((flags & NAME_MASK_NUMBER)
